@@ -2,17 +2,19 @@ import datetime
 import pytz
 import decimal
 import psycopg2
-from psycopg2 import sql
 import copy
 import json
 import singer
 import singer.metadata as metadata
+
+from psycopg2 import sql
 from singer import utils, get_bookmark
 from dateutil.parser import parse
 from functools import reduce
 
 import tap_postgres.db as post_db
 import tap_postgres.sync_strategies.common as sync_common
+from tap_postgres.stream_utils import refresh_streams_schema
 
 LOGGER = singer.get_logger('tap_postgres')
 
@@ -22,6 +24,9 @@ UPDATE_BOOKMARK_PERIOD = 10000
 class ReplicationSlotNotFoundError(Exception):
     """Custom exception when replication slot not found"""
 
+class UnsupportedPayloadKindError(Exception):
+    """Custom exception when waljson payload is not insert, update nor delete"""
+
 
 # pylint: disable=invalid-name,missing-function-docstring,too-many-branches,too-many-statements,too-many-arguments
 def get_pg_version(conn_info):
@@ -98,9 +103,10 @@ def fetch_current_lsn(conn_config):
             return lsn_to_int(current_lsn)
 
 
-def add_automatic_properties(stream, conn_config):
+def add_automatic_properties(stream, debug_lsn: bool = False):
     stream['schema']['properties']['_sdc_deleted_at'] = {'type': ['null', 'string'], 'format': 'date-time'}
-    if conn_config.get('debug_lsn'):
+
+    if debug_lsn:
         LOGGER.debug('debug_lsn is ON')
         stream['schema']['properties']['_sdc_lsn'] = {'type': ['null', 'string']}
     else:
@@ -295,7 +301,7 @@ def row_to_singer_message(stream, row, version, columns, time_extracted, md_map,
 
 
 # pylint: disable=unused-argument,too-many-locals
-def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
+def consume_message(streams, state, msg, time_extracted, conn_info):
     # Strip leading comma generated by write-in-chunks and parse valid JSON
     try:
         payload = json.loads(msg.payload.lstrip(','))
@@ -304,56 +310,58 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
 
     lsn = msg.data_start
 
-    streams_lookup = {}
-    for s in streams:
-        streams_lookup[s['tap_stream_id']] = s
+    streams_lookup = {s['tap_stream_id']: s for s in streams}
 
     tap_stream_id = post_db.compute_tap_stream_id(payload['schema'], payload['table'])
     if streams_lookup.get(tap_stream_id) is None:
         return state
 
     target_stream = streams_lookup[tap_stream_id]
+
+    if payload['kind'] not in {'insert', 'update', 'delete'}:
+        raise UnsupportedPayloadKindError("unrecognized replication operation: {}".format(payload['kind']))
+
+    # Get the additional fields in payload that are not in schema properties:
+    # only inserts and updates have the list of columns that can be used to detect any different in columns
+    diff = set()
+    if payload['kind'] in {'insert', 'update'}:
+        diff = set(payload['columnnames']).difference(target_stream['schema']['properties'].keys())
+
+    # if there is new columns in the payload that are not in the schema properties then refresh the stream schema
+    if diff:
+        LOGGER.info('Detected new columns "%s", refreshing schema of stream %s', diff, target_stream['stream'])
+        # encountered a column that is not in the schema
+        # refresh the stream schema and metadata by running discovery
+        refresh_streams_schema(conn_info, [target_stream])
+
+        # add the automatic properties back to the stream
+        add_automatic_properties(target_stream, conn_info.get('debug_lsn', False))
+
+        # publish new schema
+        sync_common.send_schema_message(target_stream, ['lsn'])
+
     stream_version = get_stream_version(target_stream['tap_stream_id'], state)
     stream_md_map = metadata.to_map(target_stream['metadata'])
 
-    desired_columns = [c for c in target_stream['schema']['properties'].keys() if sync_common.should_sync_column(
-        stream_md_map, c)]
+    desired_columns = {c for c in target_stream['schema']['properties'].keys() if sync_common.should_sync_column(
+        stream_md_map, c)}
 
-    if payload['kind'] == 'insert':
+    if payload['kind'] in {'insert', 'update'}:
         col_names = []
         col_vals = []
+
         for idx, col in enumerate(payload['columnnames']):
-            if col in set(desired_columns):
+            if col in desired_columns:
                 col_names.append(col)
                 col_vals.append(payload['columnvalues'][idx])
 
-        col_names = col_names + ['_sdc_deleted_at']
-        col_vals = col_vals + [None]
-        if conn_info.get('debug_lsn'):
-            col_names = col_names + ['_sdc_lsn']
-            col_vals = col_vals + [str(lsn)]
-        record_message = row_to_singer_message(target_stream,
-                                               col_vals,
-                                               stream_version,
-                                               col_names,
-                                               time_extracted,
-                                               stream_md_map,
-                                               conn_info)
-
-    elif payload['kind'] == 'update':
-        col_names = []
-        col_vals = []
-        for idx, col in enumerate(payload['columnnames']):
-            if col in set(desired_columns):
-                col_names.append(col)
-                col_vals.append(payload['columnvalues'][idx])
-
-        col_names = col_names + ['_sdc_deleted_at']
-        col_vals = col_vals + [None]
+        col_names.append('_sdc_deleted_at')
+        col_vals.append(None)
 
         if conn_info.get('debug_lsn'):
-            col_vals = col_vals + [str(lsn)]
-            col_names = col_names + ['_sdc_lsn']
+            col_names.append('_sdc_lsn')
+            col_vals.append(str(lsn))
+
         record_message = row_to_singer_message(target_stream,
                                                col_vals,
                                                stream_version,
@@ -366,15 +374,17 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
         col_names = []
         col_vals = []
         for idx, col in enumerate(payload['oldkeys']['keynames']):
-            if col in set(desired_columns):
+            if col in desired_columns:
                 col_names.append(col)
                 col_vals.append(payload['oldkeys']['keyvalues'][idx])
 
-        col_names = col_names + ['_sdc_deleted_at']
-        col_vals = col_vals + [singer.utils.strftime(time_extracted)]
+        col_names.append('_sdc_deleted_at')
+        col_vals.append(singer.utils.strftime(time_extracted))
+
         if conn_info.get('debug_lsn'):
-            col_vals = col_vals + [str(lsn)]
-            col_names = col_names + ['_sdc_lsn']
+            col_names.append('_sdc_lsn')
+            col_vals.append(str(lsn))
+
         record_message = row_to_singer_message(target_stream,
                                                col_vals,
                                                stream_version,
@@ -383,9 +393,6 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
                                                stream_md_map,
                                                conn_info)
 
-    else:
-        raise Exception("unrecognized replication operation: {}".format(payload['kind']))
-
     singer.write_message(record_message)
     state = singer.write_bookmark(state, target_stream['tap_stream_id'], 'lsn', lsn)
 
@@ -544,7 +551,7 @@ def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
                 LOGGER.info('Breaking - reached max_run_seconds of %i', max_run_seconds)
                 break
 
-            state = consume_message(logical_streams, state, msg, time_extracted, conn_info, end_lsn)
+            state = consume_message(logical_streams, state, msg, time_extracted, conn_info)
 
             # When using wal2json with write-in-chunks, multiple messages can have the same lsn
             # This is to ensure we only flush to lsn that has completed entirely
